disp(' ')
disp(' cross_psd.m   ver 2.9  December 5, 2012 ')
disp(' by Tom Irvine  Email: tom@vibrationdata.com ')
disp(' ')
disp(' This program calculates the cross-power spectral density ')
disp(' and coherence of two time histories. ')
disp(' ')
%
close all;
%
clear n;
clear Y;
clear amp;
clear tim;
clear N;
clear df;
clear dt;
clear Mag;
clear mag_seg;
clear amp_seg;
clear full;
clear freq;
clear ss;
clear seg;
clear i_seg;
clear rlab;
clear ylab;
%
clear t1;
clear t2;
clear A;
clear B;
%
clear PSD_A;
clear PSD_B;
%
clear CPSD;
clear CPSD_mag;
clear CPSD_phase;
clear CPSD_m;
clear CPSD_p;
%
clear COH;
%
fig_num=1;
%
disp(' ');
disp(' Select input method: ');
disp('  1=Signals are in two separate arrays.  Each with time(sec) & amp ');
disp('  2=Signals are in a common array.  time(sec) & amp & amp ');
im=input(' ');
%
iflag=0;
%
if(im==1)
    disp(' ');
    disp(' Signal A ');
    disp(' ');
    [t1,A,dt1,sr1,tmx1,tmi1,n1,~]=enter_time_history();
    disp(' ');
    disp(' Signal B ');
    disp(' ');    
    [t2,B,dt2,sr2,tmx2,tmi2,n2,ncontinue]=enter_time_history();
%
    if(tmx1 ~= tmx2 || tmi1 ~= tmi2 || n1 ~= n2 )
       disp(' Error:  time columns must be the same ');
       iflag=1;
    else
       dt=dt1;
       sr=sr1;
       t=t1;
       n=n1;   
       tmx=tmx1;
       tmi=tmi1;
    end
%
else
    [t,A,B,dt,sr,tmx,tmi,n,ncontinue]=enter_time_history_three();
end
%
if(ncontinue==1)
%
    [rlab,ylab,iam]=PSD_amplitude_metric();
%
    disp(' ')
    disp(' mean removal? ');
    mr_choice=input(' 1=yes   2=no  ' );
    disp(' ')
%
    disp(' ')
    disp(' Select Window ')
    h_choice = input(' 1=Rectangular 2=Hanning  ');
%
%%%%%%%%%%%%%%%  advise
%
    [df,mmm,NW]=PSD_advise(dt,n);
%
%%%  begin overlap
%
    mH=((mmm/2)-1);
%
    full=zeros(mH,1);
    mag_seg=zeros(mH,1);
%
    nov=0;
%
    clear amp_seg_A;
    clear amp_seg_B;
%
    CPSD=zeros(mmm,1); 
    PSD_A=zeros(mmm,1); 
    PSD_B=zeros(mmm,1); 
%
    for ijk=1:(2*NW-1)
%
        amp_seg_A=zeros(mmm,1);
        amp_seg_A(1:mmm)=A((1+ nov):(mmm+ nov));  
%
        amp_seg_B=zeros(mmm,1);
        amp_seg_B(1:mmm)=B((1+ nov):(mmm+ nov));  
%
        nov=nov+fix(mmm/2);
%
        [complex_FFT_A]=CFFT_core(amp_seg_A,mmm,mH,mr_choice,h_choice);
        [complex_FFT_B]=CFFT_core(amp_seg_B,mmm,mH,mr_choice,h_choice);        
%
        CPSD=CPSD+(conj(complex_FFT_A)).*complex_FFT_B/df;   % two-sided
        PSD_A=PSD_A+(conj(complex_FFT_A)).*complex_FFT_A/df;
        PSD_B=PSD_B+(conj(complex_FFT_B)).*complex_FFT_B/df;
%
    end
%
    den=df*(2*NW-1);
    CPSD=CPSD/den;
    PSD_A=PSD_A/den;
    PSD_B=PSD_B/den; 
%
    CPSD_mag=abs(CPSD);
    CPSD_phase=(180/pi)*atan2(imag(CPSD),real(CPSD));
%
    COH=zeros(mH,1);
    for i=1:mH
        COH(i)=CPSD_mag(i)^2/( PSD_A(i)*PSD_B(i) );
    end    
%
    fmax=(mH-1)*df;
    freq=linspace(0,fmax,mH);
 %   
    clear sum;
    ms=sum(CPSD_mag);
%
    rms=sqrt(ms*df);
%
    disp(' ');
    out4 = sprintf(' Overall RMS = %10.3g ',rms);
    out5 = sprintf(' Three Sigma = %10.3g ',3*rms);
    disp(out4)
    disp(out5)
    disp(' ');
%
    CPSD_m(1)=CPSD_mag(1);
    CPSD_m(2:mH)=2*CPSD_mag(2:mH);
%
    rms=sqrt(df*sum(CPSD_m));
%
    CPSD_p=CPSD_phase(1:mH);
%
    figure(fig_num);
    fig_num=fig_num+1;
%
    f1=10^(ceil(log10(df)));
    f2=fmax;
%
    figure(fig_num);
    fig_num=fig_num+1;
    subplot(3,1,1);
    plot(freq,CPSD_p);
    out1=sprintf('Cross Power Spectral Density, Overall level = %8.4g %s',rms,rlab);
    title(out1);
    grid on;
    ylabel('Phase (deg)');
    axis([f1,f2,-180,180]);
    set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log','YScale','lin','ytick',[-180,-90,0,90,180]);
%
    subplot(3,1,[2 3]);
    plot(freq,CPSD_m);
    grid on;
    xlabel('Frequency(Hz)');
    out1=sprintf('%s',ylab);
    ylabel(out1);
    set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log',...
         'YScale','log');
%
    ymax=10^(ceil(log10(max(CPSD_m))));
    ymin=10^(floor(log10(min(CPSD_m))));
%
    if(ymin<ymax/10000)
        ymin=ymax/10000;
    end
%
    axis([f1,f2,ymin,ymax]); 
%%%
    figure(fig_num);
    fig_num=fig_num+1;
    plot(freq,COH);
    grid on;
    set(gca,'MinorGridLineStyle','none','GridLineStyle',':','XScale','log',...
         'YScale','lin');
    xlabel('Frequency(Hz)'); 
    ylabel('(\gamma_x_y)^2'); 
    title('Coherence');
%%%
    freq=fix_size(freq);
    CPSD_m=fix_size(CPSD_m);
    CPSD_p=fix_size(CPSD_p);
%
    [xmax,fmax]=find_max([freq CPSD_m]);
%
    out5 = sprintf('\n Peak occurs at %10.5g Hz ',fmax);
    disp(out5)
%
    disp(' ');
    disp(' Output array:  cross_spectral_density ');
%
    cross_spectral_density=[freq CPSD_m CPSD_p];
%
end